﻿using AbstractSoftwareInstallationContracts.BusinessLogicsContracts;
using AbstractSoftwareInstallationContracts.BindingModels;
using AbstractSoftwareInstallationContracts.SearchModels;
using AbstractSoftwareInstallationContracts.ViewModels;
using AbstractSoftwareInstallationContracts.StoragesContracts;
using Microsoft.Extensions.Logging;
using AbstractSoftwareInstallationDataModels;

namespace AbstractSoftwareInstallationBusinessLogic.BusinessLogic
{
    public class OrderLogic : IOrderLogic
    {
        private readonly ILogger _logger;
        private readonly IOrderStorage _orderStorage;
        public OrderLogic(ILogger<OrderLogic> logger, IOrderStorage OrderStorage)
        {
            _logger = logger;
            _orderStorage = OrderStorage;
        }
        private void CheckModel(OrderBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (model.PackageId < 0)
            {
                throw new ArgumentNullException("Некорректный идентификатор у пакета", nameof(model.PackageId));
            }
            if (model.ClientId < 0)
            {
                throw new ArgumentNullException("Некорректный идентификатор у клиента", nameof(model.ClientId));
            }
            if (model.Count <= 0)
            {
                throw new ArgumentNullException("Количество пакетов в заказе должно быть больше 0", nameof(model.Count));
            }
            if (model.Sum <= 0)
            {
                throw new ArgumentNullException("Сумма заказа должна быть больше 0", nameof(model.Sum));
            }
            _logger.LogInformation("Order. OrderID:{Id}. Sum:{ Sum}. PackageId: { PackageId}. ClientId: { ClientId}", model.Id, model.Sum, model.PackageId, model.ClientId);
        }

        public List<OrderViewModel>? ReadList(OrderSearchModel? model)
        {
            _logger.LogInformation("ReadList. Orderid:{ Id}", model?.Id);
            var list = model == null ? _orderStorage.GetFullList() : _orderStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        public bool CreateOrder(OrderBindingModel model)
        {
            CheckModel(model);
            if (model.Status != OrderStatus.Неизвестен)
            { 
                _logger.LogWarning("Order status is incorrect");
                return false;
            }
            model.Status = OrderStatus.Принят;
            if (_orderStorage.Insert(model) == null)
            {
                model.Status = OrderStatus.Неизвестен;
                _logger.LogWarning("Failed to insert order into a storage");
                return false;
            }
            return true;
        }
        public bool StatusUpdate(OrderBindingModel rawModel, OrderStatus _newStatus)
        {
            var viewModel = _orderStorage.GetElement(new OrderSearchModel { Id = rawModel.Id });
            if (viewModel == null)
            {
                _logger.LogWarning("Order model not found");
                return false;
            }
            OrderBindingModel model = new OrderBindingModel
            {
                Id = viewModel.Id,
                PackageId = viewModel.PackageId,
                Status = viewModel.Status,
                DateCreate = viewModel.DateCreate,
                DateImplement = viewModel.DateImplement,
                Count = viewModel.Count,
                Sum = viewModel.Sum,
                ImplementerId = viewModel.ImplementerId
            };
            if (rawModel.ImplementerId.HasValue)
            {
                model.ImplementerId = rawModel.ImplementerId;
            }
            if (viewModel.Status + 1 != _newStatus)
            {
                _logger.LogWarning("Status update to " + _newStatus.ToString() + " operation failed. Order status incorrect.");
                return false;
            }
            rawModel.Status = _newStatus;
            if (rawModel.Status == OrderStatus.Выдан) rawModel.DateImplement = DateTime.Now;
            else
            {
                rawModel.DateImplement = viewModel.DateImplement;
            }
            CheckModel(rawModel, false);
            if (_orderStorage.Update(rawModel) == null)
            {
                rawModel.Status--;
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }
        public OrderViewModel? ReadElement(OrderSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. Id:{ Id}", model.Id);
            var element = _orderStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }
        public bool TakeOrderInWork(OrderBindingModel model)
        {
            return StatusUpdate(model, OrderStatus.Выполняется);
        }

        public bool DeliveryOrder(OrderBindingModel model)
        {
            return StatusUpdate(model, OrderStatus.Выдан);
        }

        public bool FinishOrder(OrderBindingModel model)
        {
            return StatusUpdate(model, OrderStatus.Готов);
        }
    }
}
